@@ -90,7 +90,7 @@ credits_t economy_getPrice( const Commodity *com, const StarSystem *sys,
  *    @param com Commodity to get price of.
  *    @param sys System to get price of commodity.
  *    @param p Spob to get price of commodity.
- *    @param tme Time to get price at, eg as retunred by ntime_get()
+ *    @param tme Time to get price at, e.g. as returned by `ntime_get()`
  *    @return The price of the commodity.
  */
 credits_t economy_getPriceAtTime( const Commodity *com, const StarSystem *sys,
@@ -154,7 +154,7 @@ credits_t economy_getPriceAtTime( const Commodity *com, const StarSystem *sys,
         commPrice->sysVariation * sin( 2. * M_PI * t / commPrice->sysPeriod ) +
         commPrice->spobVariation *
            sin( 2. * M_PI * t / commPrice->spobPeriod ) );
-   return (credits_t)( price + 0.5 ); /* +0.5 to round */
+   return (credits_t)MAX( 1, round( price ) ); /* +0.5 to round */
 }
 
 /**
@@ -174,6 +174,18 @@ int economy_getAverageSpobPrice( const Commodity *com, const Spob *p,
    int             i, k;
    CommodityPrice *commPrice;
 
+   double      price_mod = 1.;
+   double     *prices    = NULL;
+   Commodity **tech      = tech_getCommodity( p->tech, &prices );
+   for ( int j = 0; j < array_size( tech ); j++ ) {
+      if ( tech[j] == com ) {
+         price_mod = prices[j];
+         break;
+      }
+   }
+   array_free( prices );
+   array_free( tech );
+
    if ( com->price_ref != NULL ) {
       const Commodity *ref = commodity_get( com->price_ref );
       if ( ref == NULL )
@@ -181,14 +193,14 @@ int economy_getAverageSpobPrice( const Commodity *com, const Spob *p,
       int ret = economy_getAverageSpobPrice( ref, p, mean, std );
       if ( !ret )
          return ret;
-      *mean = (credits_t)( (double)*mean * com->price_mod + 0.5 );
+      *mean = (credits_t)( (double)*mean * com->price_mod + 0.5 ) * price_mod;
       *std  = ( *std * com->price_mod );
       return 0;
    }
 
    /* Constant price. */
    if ( commodity_isFlag( com, COMMODITY_FLAG_PRICE_CONSTANT ) ) {
-      *mean = com->price;
+      *mean = com->price * price_mod;
       *std  = 0.;
       return 0;
    }
@@ -223,8 +235,7 @@ int economy_getAverageSpobPrice( const Commodity *com, const Spob *p,
    }
    commPrice = &p->commodityPrice[i];
    if ( commPrice->cnt > 0 ) {
-      *mean = (credits_t)( commPrice->sum / commPrice->cnt +
-                           0.5 ); /* +0.5 to round*/
+      *mean = (credits_t)round( commPrice->sum / commPrice->cnt * price_mod );
       *std  = ( sqrt( commPrice->sum2 / commPrice->cnt -
                       ( commPrice->sum * commPrice->sum ) /
                          ( commPrice->cnt * commPrice->cnt ) ) );
@@ -253,9 +264,9 @@ int economy_getAveragePrice( const Commodity *com, credits_t *mean,
       if ( ref == NULL )
          return -1;
       int ret = economy_getAveragePrice( ref, mean, std );
-      *mean   = (credits_t)( (double)*mean * com->price_mod + 0.5 );
+      *mean   = (credits_t)round( (double)*mean * com->price_mod );
       *std    = ( *std * com->price_mod );
-      return ( (double)ret * com->price_mod + 0.5 );
+      return round( (double)ret * com->price_mod );
    }
 
    /* Constant price. */
@@ -289,7 +300,7 @@ int economy_getAveragePrice( const Commodity *com, credits_t *mean,
    if ( cnt > 0 ) {
       av /= cnt;
       av2   = sqrt( av2 / cnt - av * av );
-      *mean = (credits_t)( av + 0.5 );
+      *mean = (credits_t)round( av );
       *std  = av2;
    } else {
       WARN( _( "Average price for commodity '%s' not known." ), com->name );
@@ -501,7 +512,7 @@ int economy_refresh( void )
 /**
  * @brief Updates the economy.
  *
- *    @param dt Deltatick in NTIME.
+ *    @param dt Delta time in NTIME.
  */
 int economy_update( unsigned int dt )
 {
@@ -606,7 +617,7 @@ void economy_destroy( void )
  *
  *    @param spob The spob to set price on.
  *    @param commodity The commodity to set the price of.
- *    @param commodityPrice Where to write the commodity price to.
+ *    @param[out] commodityPrice Where to write the commodity price to.
  *    @return 0 on success.
  */
 static int economy_calcPrice( Spob *spob, Commodity *commodity,
@@ -620,6 +631,14 @@ static int economy_calcPrice( Spob *spob, Commodity *commodity,
    if ( !spob_hasService( spob, SPOB_SERVICE_COMMODITY ) )
       return 0;
 
+   /* Constant price. */
+   if ( commodity_isFlag( commodity, COMMODITY_FLAG_PRICE_CONSTANT ) ) {
+      commodityPrice->price = commodity->price;
+      commodityPrice->sum   = commodity->price;
+      commodityPrice->sum2  = 0.;
+      return 0;
+   }
+
    /* Check the faction is not NULL.*/
    if ( spob->presence.faction == -1 ) {
       WARN( _( "Spob '%s' appears to have commodity '%s' defined, but no "
@@ -864,7 +883,7 @@ static void economy_calcUpdatedCommodityPrice( StarSystem *sys )
 }
 
 /**
- * @brief Initialises commodity prices for the sinusoidal economy model.
+ * @brief Initializes commodity prices for the sinusoidal economy model.
  *
  */
 void economy_initialiseCommodityPrices( void )
@@ -981,14 +1000,7 @@ void economy_clearKnown( void )
    for ( int i = 0; i < array_size( systems_stack ); i++ ) {
       StarSystem *sys = &systems_stack[i];
       for ( int j = 0; j < array_size( sys->spobs ); j++ ) {
-         Spob *p = sys->spobs[j];
-         for ( int k = 0; k < array_size( p->commodityPrice ); k++ ) {
-            CommodityPrice *cp = &p->commodityPrice[k];
-            cp->cnt            = 0;
-            cp->sum            = 0;
-            cp->sum2           = 0;
-            cp->updateTime     = 0;
-         }
+         economy_clearSingleSpob( sys->spobs[j] );
       }
    }
    for ( int i = 0; i < array_size( commodity_stack ); i++ )
@@ -1004,8 +1016,8 @@ void economy_clearSingleSpob( Spob *p )
    for ( int k = 0; k < array_size( p->commodityPrice ); k++ ) {
       CommodityPrice *cp = &p->commodityPrice[k];
       cp->cnt            = 0;
-      cp->sum            = 0;
-      cp->sum2           = 0;
+      cp->sum            = 0.;
+      cp->sum2           = 0.;
       cp->updateTime     = 0;
    }
 }
@@ -1074,8 +1086,9 @@ int economy_sysLoad( xmlNodePtr parent )
             } else if ( xml_isNode( cur, "lastPurchase" ) ) {
                xmlr_attr_strd( cur, "name", str );
                if ( str ) {
-                  Commodity *c         = commodity_get( str );
-                  c->lastPurchasePrice = xml_getLong( cur );
+                  Commodity *c = commodity_get( str );
+                  if ( c != NULL )
+                     c->lastPurchasePrice = xml_getLong( cur );
                   free( str );
                }
             }
@@ -1111,26 +1124,27 @@ int economy_sysSave( xmlTextWriterPtr writer )
          int   doneSpob = 0;
          for ( int k = 0; k < array_size( p->commodities ); k++ ) {
             CommodityPrice *cp = &p->commodityPrice[k];
-            if ( cp->cnt >
-                 0 ) { /* Player has seen this commodity at this location */
-               if ( doneSys == 0 ) {
-                  doneSys = 1;
-                  xmlw_startElem( writer, "system" );
-                  xmlw_attr( writer, "name", "%s", sys->name );
-               }
-               if ( doneSpob == 0 ) {
-                  doneSpob = 1;
-                  xmlw_startElem( writer, "spob" );
-                  xmlw_attr( writer, "name", "%s", p->name );
-               }
-               xmlw_startElem( writer, "commodity" );
-               xmlw_attr( writer, "name", "%s", p->commodities[k]->name );
-               xmlw_attr( writer, "sum", "%f", cp->sum );
-               xmlw_attr( writer, "sum2", "%f", cp->sum2 );
-               xmlw_attr( writer, "cnt", "%d", cp->cnt );
-               xmlw_attr( writer, "time", "%" PRIu64, cp->updateTime );
-               xmlw_endElem( writer ); /* commodity */
+            if ( cp->cnt <= 0 )
+               continue;
+            /* Player has seen this commodity at this location */
+
+            if ( doneSys == 0 ) {
+               doneSys = 1;
+               xmlw_startElem( writer, "system" );
+               xmlw_attr( writer, "name", "%s", sys->name );
             }
+            if ( doneSpob == 0 ) {
+               doneSpob = 1;
+               xmlw_startElem( writer, "spob" );
+               xmlw_attr( writer, "name", "%s", p->name );
+            }
+            xmlw_startElem( writer, "commodity" );
+            xmlw_attr( writer, "name", "%s", p->commodities[k]->name );
+            xmlw_attr( writer, "sum", "%f", cp->sum );
+            xmlw_attr( writer, "sum2", "%f", cp->sum2 );
+            xmlw_attr( writer, "cnt", "%d", cp->cnt );
+            xmlw_attr( writer, "time", "%" PRIu64, cp->updateTime );
+            xmlw_endElem( writer ); /* commodity */
          }
          if ( doneSpob == 1 )
             xmlw_endElem( writer ); /* spob */
