@@ -19,21 +19,20 @@
 #include "nlua_tex.h"
 #include "nlua_tk.h"
 
-static nlua_env naevpedia_env = LUA_NOREF; /**< Naevpedia environment. */
+static nlua_env *naevpedia_env = NULL; /**< Naevpedia environment. */
 
 /**
- * @brief Try to open naevpedia path.
+ * @brief Try to open Naevpedia path.
  */
 int naevpedia_open( const char *path )
 {
    char buf[STRMAX_SHORT];
    int  status;
 
-   if ( naevpedia_env == LUA_NOREF ) {
+   if ( naevpedia_env == NULL ) {
       naevpedia_env = nlua_newEnv( "naevpedia" );
       nlua_loadStandard( naevpedia_env );
       nlua_loadTex( naevpedia_env );
-      nlua_loadCol( naevpedia_env );
       nlua_loadBackground( naevpedia_env );
       nlua_loadMusic( naevpedia_env );
       nlua_loadTk( naevpedia_env );
