@@ -71,7 +71,7 @@ static const luaL_Reg asteroidL_methods[] = {
  *    @param env Lua environment.
  *    @return 0 on success.
  */
-int nlua_loadAsteroid( nlua_env env )
+int nlua_loadAsteroid( nlua_env *env )
 {
    nlua_register( env, ASTEROID_METATABLE, asteroidL_methods, 1 );
    return 0;
@@ -207,7 +207,7 @@ static int asteroidL_eq( lua_State *L )
 
 /**
  * @brief Gets the asteroid's current (translated) name or notes it is
- * inexistent.
+ * non-existent.
  *
  * @usage tostring(a)
  *
@@ -356,7 +356,7 @@ static int asteroidL_get( lua_State *L )
  * @brief Checks to see if an asteroid exists.
  *
  *    @luatparam Asteroid a Asteroid to check to see if exists.
- *    @luatreturn boolean true if te asteroid exists, false otherwise.
+ *    @luatreturn boolean true if the asteroid exists, false otherwise.
  * @luafunc exists
  */
 static int asteroidL_exists( lua_State *L )
@@ -388,8 +388,8 @@ static int asteroidL_exists( lua_State *L )
  * @brief Gets the state of an asteroid.
  *
  *    @luatparam Asteroid a Asteroid to check state of.
- *    @luatreturn string State of the asteroid. Can be one of "FG", "XB", "BX",
- * "XX_TO_BG", "FG_TO_BG", "BG_TO_FG", "BG_TO_XX", or "XX".
+ *    @luatreturn string State of the asteroid. Can be one of `FG`, `XB`, `BX`,
+ * `XX_TO_BG`, `FG_TO_BG`, `BG_TO_FG`, `BG_TO_XX`, or `XX`.
  * @luafunc state
  */
 static int asteroidL_state( lua_State *L )
@@ -430,9 +430,9 @@ static int asteroidL_state( lua_State *L )
  * @brief Sets the state of an asteroid.
  *
  *    @luatparam Asteroid a Asteroid to set state of.
- *    @luatparam string s State to set. Has to be one of "FG", "XB", "BX",
- * "XX_TO_BG", "FG_TO_BG", "BG_TO_FG", "BG_TO_XX", or "XX".
- * @luafunc state
+ *    @luatparam string s State to set. Has to be one of `FG`, `XB`, `BX`,
+ * `XX_TO_BG`, `FG_TO_BG`, `BG_TO_FG`, `BG_TO_XX`, or `XX`.
+ * @luafunc setState
  */
 static int asteroidL_setState( lua_State *L )
 {
@@ -597,7 +597,7 @@ static int asteroidL_armour( lua_State *L )
  *    @luatparam Asteroid a Asteroid to set armour of.
  *    @luatparam number Amount to set the armour to (negative values will
  * explode it).
- * @luafunc setTimer
+ * @luafunc setArmour
  */
 static int asteroidL_setArmour( lua_State *L )
 {
