@@ -122,7 +122,7 @@ static unsigned int hookL_generic( lua_State *L, const char *stack, double ms,
  *    @param env Lua environment.
  *    @return 0 on success.
  */
-int nlua_loadHook( nlua_env env )
+int nlua_loadHook( nlua_env *env )
 {
    nlua_register( env, "hook", hookL_methods, 0 );
    return 0;
@@ -139,11 +139,11 @@ int nlua_loadHook( nlua_env env )
  *
  * Example usage would be:
  * @code
- * function penter( arg )
+ * function penter( argument )
  *    -- Function to run when player enters a system
  * end
  *
- * hookid = hook.enter( "penter", 5 )
+ * hook_id = hook.enter( "penter", 5 )
  * @endcode
  *
  * @luamod hook
@@ -177,7 +177,7 @@ static int hookL_rm( lua_State *L )
  */
 static int hookL_setarg( unsigned int hook, int ind )
 {
-   nlua_env env = hook_env( hook );
+   nlua_env *env = hook_env( hook );
 
    /* Create if necessary the actual hook argument table. */
    nlua_getenv( naevL, env, "mem" );           /* t */
@@ -200,9 +200,9 @@ static int hookL_setarg( unsigned int hook, int ind )
  */
 void hookL_unsetarg( unsigned int hook )
 {
-   nlua_env env = hook_env( hook );
+   nlua_env *env = hook_env( hook );
 
-   if ( env == LUA_NOREF )
+   if ( env == NULL )
       return;
 
    nlua_getenv( naevL, env, "mem" );        /* t */
@@ -223,9 +223,9 @@ void hookL_unsetarg( unsigned int hook )
  */
 int hookL_getarg( unsigned int hook )
 {
-   nlua_env env = hook_env( hook );
+   nlua_env *env = hook_env( hook );
 
-   if ( env == LUA_NOREF )
+   if ( env == NULL )
       return 0;
 
    nlua_getenv( naevL, env, "mem" );        /* t */
@@ -312,13 +312,13 @@ static unsigned int hookL_generic( lua_State *L, const char *stack, double sec,
  *
  * Can also be used to hook the various subparts of the landing menu. Possible
  * targets for where are:<br />
- *   - "land" - when landed (default with no parameter )<br />
- *   - "outfits" - when visited outfitter<br />
- *   - "shipyard" - when visited shipyard<br />
- *   - "bar" - when visited bar<br />
- *   - "mission" - when visited mission computer<br />
- *   - "commodity" - when visited commodity exchange<br />
- *   - "equipment" - when visiting equipment place<br />
+ *   - `"land"` - when landed (default with no parameter )<br />
+ *   - `"outfits"` - when visited outfitter<br />
+ *   - `"shipyard"` - when visited shipyard<br />
+ *   - `"bar"` - when visited bar<br />
+ *   - `"mission"` - when visited mission computer<br />
+ *   - `"commodity"` - when visited commodity exchange<br />
+ *   - `"equipment"` - when visiting equipment place<br />
  *
  * @usage hook.land( "my_function" ) -- Land calls my_function
  * @usage hook.land( "my_function", "equipment" ) -- Calls my_function at
@@ -350,13 +350,13 @@ static int hookL_land( lua_State *L )
  *
  * Can also be used to hook the various subparts of the info menu. Possible
  * targets for where are:<br />
- *   - "main"<br />
- *   - "ship"<br />
- *   - "weapons"<br />
- *   - "cargo"<br />
- *   - "mission"<br />
- *   - "standing"<br />
- *   - "shiplog"<br />
+ *   - `"main"`<br />
+ *   - `"ship"`<br />
+ *   - `"weapons"`<br />
+ *   - `"cargo"`<br />
+ *   - `"mission"`<br />
+ *   - `"standing"`<br />
+ *   - `"shiplog"`<br />
  *
  * @usage hook.info( "my_function" ) -- Info calls my_function
  * @usage hook.info( "my_function", "equipment" ) -- Calls my_function at
@@ -634,7 +634,7 @@ static int hookL_commjettison( lua_State *L )
 }
 
 /**
- * @brief Hooks the function to the player gatehring any sort of commodity in
+ * @brief Hooks the function to the player gathering any sort of commodity in
  * space.
  *
  * The hook receives the commodity and the quantity being gathered.
@@ -686,7 +686,7 @@ static int hookL_outfitsell( lua_State *L )
 }
 
 /**
- * @brief Hooks the function to the player equipping or deequipping any outfit.
+ * @brief Hooks the function to the player equipping or unequipping any outfit.
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -738,8 +738,11 @@ static int hookL_shipsell( lua_State *L )
 /**
  * @brief Hooks the function to the player swapping their ship.
  *
- * The hook receives the name of the ship swapped to and the name of the ship
- * swapped from (if applicable).
+ * The hook receives four parameters: the name of the ship swapped to, the ship
+ * swapped to, the name of the ship swapped from, and the ship swapped from (if
+ * applicable).
+ *
+ * @usage function my_hook( toshipname, toship, fromshipname, fromship ) ... end
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -756,9 +759,9 @@ static int hookL_shipswap( lua_State *L )
 /**
  * @brief Hooks the function to the player pressing any input.
  *
- * It returns the name of the key being pressed like "accel" and whether or not
- * it's a press.<br/> <br/> Functions should be in format:<br/> function f(
- * inputname, inputpress, args )
+ * It returns the name of the key being pressed like `"accel"` and whether or
+ * not it's a press. <br/> <br/> Functions should be in format: <br/>
+ * `function f(inputname, inputpress, args)`
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -796,7 +799,7 @@ static int hookL_mouse( lua_State *L )
  *
  * The parameters passed to the function are faction whose standing is being
  * changed and the amount changed:<br/>
- * function f( faction, change, system, secondary, primary_fct, args )
+ * function `f( faction, change, system, secondary, primary_fct, args)`
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -815,8 +818,8 @@ static int hookL_standing( lua_State *L )
  * the likes.
  *
  * The parameters passed to the function are the type which can be one of:<br/>
- * - "spob" <br/>
- * - "jump" <br/>
+ * - `"spob"` <br/>
+ * - `"jump"` <br/>
  * and the actual spob or jump point discovered with the following format: <br/>
  * function f( type, discovery )
  *
@@ -855,7 +858,8 @@ static int hookL_asteroidScan( lua_State *L )
  * player.pay() (the Lua function only).
  *
  * The amount paid (or taken from the player) and reason (which is nil by
- * default) is passed as a parameter:<br/> function f( amount, reason, args )
+ * default) is passed as a parameter:
+ * <br/> `function f( amount, reason, args )`
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -896,8 +900,8 @@ static int hookL_safe( lua_State *L )
  * it continues forever.
  *
  * The current delta-tick (time passed in game) and real delta-tick (independent
- * of game status) are passed as parameters:<br/> function f( dt, real_dt, args
- * )
+ * of game status) are passed as parameters:
+ * <br/> `function f( dt, real_dt, args )`
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
  *    @luaparam arg Argument to pass to hook.
@@ -928,7 +932,7 @@ static int hookL_renderbg( lua_State *L )
 }
 
 /**
- * @brief Hook that runs during rendering the foreground (just below the gui
+ * @brief Hook that runs during rendering the foreground (just below the GUI
  * stuff). Meant to be only for rendering things.
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
@@ -944,7 +948,7 @@ static int hookL_renderfg( lua_State *L )
 }
 
 /**
- * @brief Hook that runs during rendering aove everything. Meant to be as an
+ * @brief Hook that runs during rendering above everything. Meant to be as an
  * alternative to doing post-processing shader effects.
  *
  *    @luatparam string funcname Name of function to run when hook is triggered.
@@ -1043,13 +1047,16 @@ static int hookL_custom( lua_State *L )
  * all pilots.<br /> <br /> DO NOT DO UNSAFE THINGS IN PILOT HOOKS. THIS MEANS
  * STUFF LIKE player.teleport(). IF YOU HAVE DOUBTS USE A "safe" HOOK.<br /> <br
  * /> These hooks all pass the pilot triggering the hook as a parameter, so they
- * should have the structure of: <p> function my_hook( pilot, arg )<br /> end
+ * should have the structure of:
+ * <p>
+ *    function my_hook( pilot, argument )<br />
+ *    end
  * </p>
  * The combat hooks also pass the pilot acting on it, so for example the pilot
  *  that disabled, attacked or killed the selected pilot. They have the
  *  following format:
  * <p>
- *    function combat_hook( pilot, attacker, arg )<br />
+ *    function combat_hook( pilot, attacker, argument )<br />
  *    end
  * </p>
  * Please note that in the case of disable or death hook the attacker may be nil
@@ -1059,20 +1066,20 @@ static int hookL_custom( lua_State *L )
  * The land and jump hooks also pass the spob or jump point the pilot is
  * landing at or jumped from, respectively:
  * <p>
- *    function land_hook( pilot, spob, arg )<br />
+ *    function land_hook( pilot, spob, argument )<br />
  *    end
  * </p>
  * <p style="margin-bottom: 0">
- *    function jump_hook( pilot, jump_point, arg )<br />
+ *    function jump_hook( pilot, jump_point, argument )<br />
  *    end
  * </p>
  * The stealth hook passes whether or not the ship is stealthing or destealthing
- * as a boolean: <p> function stealth_hook( pilot, status, arg )<br /> end
+ * as a boolean: <p> function stealth_hook( pilot, status, argument )<br /> end
  * </p>
  *    @luatparam Pilot|nil pilot Pilot identifier to hook (or nil for all).
  *    @luatparam string type One of the supported hook types.
  *    @luatparam string funcname Name of function to run when hook is triggered.
- *    @luaparam arg Argument to pass to hook.
+ *    @luaparam argument Argument to pass to hook.
  *    @luatreturn number Hook identifier.
  * @luafunc pilot
  */
