@@ -10,7 +10,7 @@
 #include <lauxlib.h>
 #include <time.h>
 
-#include "SDL_timer.h"
+#include <SDL3/SDL_timer.h>
 
 #include "naev.h"
 /** @endcond */
@@ -33,7 +33,6 @@
 #include "pause.h"
 #include "player.h"
 #include "plugin.h"
-#include "semver.h"
 
 static int cache_table = LUA_NOREF; /* No reference. */
 
@@ -72,11 +71,13 @@ static int naevL_pause( lua_State *L );
 static int naevL_unpause( lua_State *L );
 static int naevL_hasTextInput( lua_State *L );
 static int naevL_setTextInput( lua_State *L );
+static int naevL_shipstats( lua_State *L );
 static int naevL_unit( lua_State *L );
 static int naevL_quadtreeParams( lua_State *L );
 static int naevL_difficulty( lua_State *L );
+static int naevL_difficultyLevel( lua_State *L );
+static int naevL_supported_image_ext( lua_State *L );
 #if DEBUGGING
-static int naevL_envs( lua_State *L );
 static int naevL_debugTrails( lua_State *L );
 static int naevL_debugCollisions( lua_State *L );
 #endif /* DEBUGGING */
@@ -116,11 +117,13 @@ static const luaL_Reg naev_methods[] = {
    { "unpause", naevL_unpause },
    { "hasTextInput", naevL_hasTextInput },
    { "setTextInput", naevL_setTextInput },
+   { "shipstats", naevL_shipstats },
    { "unit", naevL_unit },
    { "quadtreeParams", naevL_quadtreeParams },
    { "difficulty", naevL_difficulty },
+   { "difficultyLevel", naevL_difficultyLevel },
+   { "supported_image_ext", naevL_supported_image_ext },
 #if DEBUGGING
-   { "envs", naevL_envs },
    { "debugTrails", naevL_debugTrails },
    { "debugCollisions", naevL_debugCollisions },
 #endif         /* DEBUGGING */
@@ -132,7 +135,7 @@ static const luaL_Reg naev_methods[] = {
  *    @param env Lua environment.
  *    @return 0 on success.
  */
-int nlua_loadNaev( nlua_env env )
+int nlua_loadNaev( nlua_env *env )
 {
    nlua_register( env, "naev", naev_methods, 0 );
 
@@ -171,42 +174,26 @@ static int naevL_version( lua_State *L )
 }
 
 /**
- * @brief Tests two semver version strings.
+ * @brief Test a version string.
  *
- *    @luatparam string v1 Version 1 to test.
- *    @luatparam string v2 Version 2 to test.
- *    @luatreturn number Positive if v1 is newer or negative if v2 is newer.
+ *    @luatparam string ver Version to test.
+ *    @luatparam string req Requirement condition.
+ *    @luatreturn boolean Whether or not the version meets the requirements.
  * @luafunc versionTest
  */
 static int naevL_versionTest( lua_State *L )
 {
-   const char *s1, *s2;
-   semver_t    sv1, sv2;
-   int         res;
-   /* Parse inputs. */
-   s1 = luaL_checkstring( L, 1 );
-   s2 = luaL_checkstring( L, 2 );
-   if ( semver_parse( s1, &sv1 ) )
-      NLUA_WARN( L, _( "Failed to parse version string '%s'!" ), s1 );
-   if ( semver_parse( s2, &sv2 ) )
-      NLUA_WARN( L, _( "Failed to parse version string '%s'!" ), s2 );
-
-   /* Check version. */
-   res = semver_compare( sv1, sv2 );
-
-   /* Cleanup. */
-   semver_free( &sv1 );
-   semver_free( &sv2 );
-
-   lua_pushinteger( L, res );
+   const char *s1 = luaL_checkstring( L, 1 );
+   const char *s2 = luaL_checkstring( L, 2 );
+   lua_pushboolean( L, naev_versionMatchReq( s1, s2 ) );
    return 1;
 }
 
 /**
  * @brief Gets the current language locale.
  *
- *    @luatreturn string Current language locale (such as "en" for English, "de"
- * for German, or "ja" for Japanese).
+ *    @luatreturn string Current language locale (such as `"en"` for English,
+ * `"de"` for German, or `"ja"` for Japanese).
  * @luafunc language
  */
 static int naevL_language( lua_State *L )
@@ -224,15 +211,16 @@ static int naevL_language( lua_State *L )
  */
 static int naevL_lastplayed( lua_State *L )
 {
-   double d = difftime( time( NULL ), player.last_played );
-   double g = difftime( time( NULL ), conf.last_played );
-   lua_pushnumber( L, d / ( 3600. * 24. ) ); /*< convert to days */
-   lua_pushnumber( L, g / ( 3600. * 24. ) ); /*< convert to days */
+   const double TO_DAYS = 1. / ( 3600. * 24. ); /* Convert to days. */
+   double       d       = difftime( time( NULL ), player.last_played );
+   double       g       = difftime( time( NULL ), conf.last_played );
+   lua_pushnumber( L, d * TO_DAYS );
+   lua_pushnumber( L, g * TO_DAYS );
    return 2;
 }
 
 /**
- * @brief Equivalent to os.date from standard Lua.
+ * @brief Equivalent to `os.date` from standard Lua.
  *
  * @luafunc date
  */
@@ -354,7 +342,7 @@ static int naevL_fps( lua_State *L )
  * @usage bindname = naev.keyGet( "accel" )
  *
  *    @luatparam string keyname Name of the keybinding to get value of. Valid
- * values are listed in src/input.c: keybind_info.
+ * values are listed in "src/input.c": `keybind_info`.
  * @luafunc keyGet
  */
 static int naevL_keyGet( lua_State *L )
@@ -373,7 +361,7 @@ static int naevL_keyGet( lua_State *L )
  *
  * @usage naev.keyEnable( "accel", false ) -- Disables the acceleration key
  *    @luatparam string keyname Name of the key to disable (for example
- * "accel").
+ * `"accel"`).
  *    @luatparam[opt=false] boolean enable Whether to enable or disable.
  * @luafunc keyEnable
  */
@@ -456,7 +444,7 @@ static int naevL_missionList( lua_State *L )
  * @usage naev.missionStart( "Some Mission" )
  *    @luatparam string misnname Name of the mission to start.
  *    @luatreturn boolean true if mission was either accepted, or started
- * without misn.finish() getting called in create.
+ * without `misn.finish()` getting called in create.
  *    @luatreturn boolean true whether or not the mission was accepted.
  * @luafunc missionStart
  */
@@ -480,7 +468,7 @@ static int naevL_missionStart( lua_State *L )
  * player.
  *
  * Note that this tests the Lua conditionals, not the create function, so it may
- * be possible that even though naev.missionTest returns true, the player can
+ * be possible that even though `naev.missionTest` returns true, the player can
  * still not start the mission.
  *
  * @usage naev.missionTest( "Some Mission" )
@@ -561,7 +549,7 @@ static int naevL_isSimulation( lua_State *L )
 
 #define PUSH_STRING( L, name, value )                                          \
    lua_pushstring( L, name );                                                  \
-   lua_pushstring( L, value );                                                 \
+   lua_pushstring( L, ( ( value ) == NULL ) ? "" : ( value ) );                \
    lua_rawset( L, -3 )
 #define PUSH_DOUBLE( L, name, value )                                          \
    lua_pushstring( L, name );                                                  \
@@ -595,9 +583,7 @@ static int naevL_conf( lua_State *L )
    PUSH_DOUBLE( L, "scalefactor", conf.scalefactor );
    PUSH_DOUBLE( L, "nebu_scale", conf.nebu_scale );
    PUSH_BOOL( L, "fullscreen", conf.fullscreen );
-   PUSH_BOOL( L, "modesetting", conf.modesetting );
    PUSH_BOOL( L, "notresizable", conf.notresizable );
-   PUSH_BOOL( L, "borderless", conf.borderless );
    PUSH_BOOL( L, "minimize", conf.minimize );
    PUSH_DOUBLE( L, "colourblind_sim", conf.colourblind_sim );
    PUSH_DOUBLE( L, "colourblind_correct", conf.colourblind_correct );
@@ -787,7 +773,6 @@ static int naevL_plugins( lua_State *L )
       STRING( author );
       STRING( version );
       STRING( description );
-      STRING( compatibility );
       STRING( mountpoint );
 
       INTEGER( priority );
@@ -808,12 +793,12 @@ static int naevL_plugins( lua_State *L )
  * @brief Opens the info menu window.
  *
  * Possible window targets are: <br />
- *  - "main" : Main window.<br />
- *  - "ship" : Ship info window.<br />
- *  - "weapons" : Weapon configuration window.<br />
- *  - "cargo" : Cargo view window.<br />
- *  - "missions" : Mission view window.<br />
- *  - "standings" : Standings view window.<br />
+ *  - `"main"` : Main window.<br />
+ *  - `"ship"` : Ship info window.<br />
+ *  - `"weapons"` : Weapon configuration window.<br />
+ *  - `"cargo"` : Cargo view window.<br />
+ *  - `"missions"` : Mission view window.<br />
+ *  - `"standings"` : Standings view window.<br />
  *
  * @usage naev.menuInfo( "ship" ) -- Opens ship tab
  *
@@ -838,17 +823,17 @@ static int naevL_menuInfo( lua_State *L )
    }
 
    /* Parse string. */
-   if ( strcasecmp( str, "main" ) == 0 )
+   if ( SDL_strcasecmp( str, "main" ) == 0 )
       window = INFO_MAIN;
-   else if ( strcasecmp( str, "ship" ) == 0 )
+   else if ( SDL_strcasecmp( str, "ship" ) == 0 )
       window = INFO_SHIP;
-   else if ( strcasecmp( str, "weapons" ) == 0 )
+   else if ( SDL_strcasecmp( str, "weapons" ) == 0 )
       window = INFO_WEAPONS;
-   else if ( strcasecmp( str, "cargo" ) == 0 )
+   else if ( SDL_strcasecmp( str, "cargo" ) == 0 )
       window = INFO_CARGO;
-   else if ( strcasecmp( str, "missions" ) == 0 )
+   else if ( SDL_strcasecmp( str, "missions" ) == 0 )
       window = INFO_MISSIONS;
-   else if ( strcasecmp( str, "standings" ) == 0 )
+   else if ( SDL_strcasecmp( str, "standings" ) == 0 )
       window = INFO_STANDINGS;
    else
       return NLUA_ERROR( L, _( "Invalid window info name '%s'." ), str );
@@ -881,7 +866,7 @@ static int naevL_menuSmall( lua_State *L )
  * @brief Checks to see if the game is paused.
  *
  *    @luatreturn boolean Whether or not the game is currently paused.
- * @luafunc pause
+ * @luafunc isPaused
  */
 static int naevL_isPaused( lua_State *L )
 {
@@ -924,7 +909,7 @@ static int naevL_unpause( lua_State *L )
  */
 static int naevL_hasTextInput( lua_State *L )
 {
-   lua_pushboolean( L, SDL_EventState( SDL_TEXTINPUT, SDL_QUERY ) == SDL_TRUE );
+   lua_pushboolean( L, SDL_EventEnabled( SDL_EVENT_TEXT_INPUT ) );
    return 1;
 }
 
@@ -946,16 +931,36 @@ static int naevL_setTextInput( lua_State *L )
       input_pos.y = luaL_checkinteger( L, 3 );
       input_pos.w = luaL_checkinteger( L, 4 );
       input_pos.h = luaL_checkinteger( L, 5 );
-      SDL_EventState( SDL_TEXTINPUT, SDL_ENABLE );
-      SDL_StartTextInput();
-      SDL_SetTextInputRect( &input_pos );
+      SDL_SetEventEnabled( SDL_EVENT_TEXT_INPUT, 1 );
+      SDL_StartTextInput( gl_screen.window );
+      SDL_SetTextInputArea( gl_screen.window, &input_pos, 0 );
    } else {
-      SDL_StopTextInput();
-      SDL_EventState( SDL_TEXTINPUT, SDL_DISABLE );
+      SDL_ClearComposition( gl_screen.window );
+      SDL_StopTextInput( gl_screen.window );
+      SDL_SetTextInputArea( gl_screen.window, NULL, 0 );
+      SDL_SetEventEnabled( SDL_EVENT_TEXT_INPUT, 0 );
    }
    return 0;
 }
 
+/**
+ * @brief Gets the translated string corresponding to an in-game unit.
+ * <ul>
+ *  <li>name: Name of the ship stat.</li>
+ *  <li>display: Untranslated display name of the ship stat, which you use to
+ * show to the player.<li> <li>unit: Untranslated unit name (if applicable,
+ * otherwise nil).</li> <li>inverted: Boolean value whether or not negative is
+ * "better" for the player.</li>
+ * </ul>
+ *    @luatreturn Table with all the ship stats information.
+ * @luafunc shipstats
+ */
+static int naevL_shipstats( lua_State *L )
+{
+   ss_exportLua( L );
+   return 1;
+}
+
 static const char *unittbl[] = {
    "time",         _UNIT_TIME,   "per_time",     _UNIT_PER_TIME, "distance",
    _UNIT_DISTANCE, "speed",      _UNIT_SPEED,    "accel",        _UNIT_ACCEL,
@@ -966,7 +971,7 @@ static const char *unittbl[] = {
 };
 /**
  * @brief Gets the translated string corresponding to an in-game unit.
- *    @luaparam[opt=nil] string str Name of the unit to get or nil to get a
+ *    @luatparam[opt=nil] string str Name of the unit to get or nil to get a
  * table with all of them.
  *    @luareturn Translated string corresponding to the unit or table of all
  * strings if no parameter is passed.
@@ -1029,21 +1034,42 @@ static int naevL_difficulty( lua_State *L )
    return 2;
 }
 
-#if DEBUGGING
 /**
- * @brief Gets a table with all the active Naev environments.
+ * @brief Gets the difficulty level settings.
  *
- * Only available only debug builds.
- *
- *    @luatreturn table Unordered table containing all the environments.
- * @luafunc envs
+ *    @luatreturn number Difficulty level settings.
+ * @luafunc difficultyLevel
  */
-static int naevL_envs( lua_State *L )
+static int naevL_difficultyLevel( lua_State *L )
 {
-   nlua_pushEnvTable( L );
+   const Difficulty *dif = difficulty_cur();
+   lua_pushnumber( L, dif->level );
    return 1;
 }
 
+/**
+ * @brief Gets the supported image file extensions (useful for searching).
+ *
+ *    @luatreturn table A table with all the supported image file extensions.
+ * @luafunc supported_image_ext
+ */
+static int naevL_supported_image_ext( lua_State *L )
+{
+   int i = 1;
+   // TODO do this properly in Rust with image-rs
+#define EXT( s )                                                               \
+   lua_pushstring( L, ( s ) );                                                 \
+   lua_rawseti( L, -2, i++ );
+   lua_newtable( L );
+   EXT( ".avif" );
+   EXT( ".webp" );
+   EXT( ".png" );
+   EXT( ".jpg" );
+   EXT( ".jpeg" );
+   return 1;
+}
+
+#if DEBUGGING
 /**
  * @brief Toggles the trail emitters.
  *
