@@ -5,8 +5,8 @@
  * @file safelanes.c
  *
  * @brief Handles factions' safe lanes through systems.
- * This implements the algorithm described in utils/lanes-generator (whitepaper
- * and much clearer Python version).
+ * This implements the algorithm described in "utils/lanes-generator"
+ * (whitepaper and much clearer Python version).
  */
 /** @cond */
 #include <math.h>
@@ -35,7 +35,7 @@
 #define I_LOVE_FORTRAN 1
 #endif
 
-#include "SDL_timer.h"
+#include <SDL3/SDL_timer.h>
 
 #include "naev.h"
 /** @endcond */
@@ -46,6 +46,7 @@
 #include "conf.h"
 #include "log.h"
 #include "union_find.h"
+#include "valgrind.h"
 
 /*
  * Global parameters.
@@ -203,13 +204,14 @@ static inline void array_push_back_edge( Edge **a, int v0, int v1 )
 /**
  * @brief Initializes the safelanes system.
  */
-void safelanes_init( void )
+int safelanes_init( void )
 {
    cholmod_start( &C );
    /* Ideally we would want to recalculate here, but since we load the first
     * save and try to use unidiffs there, we instead defer the safe lane
     * computation to only if necessary after loading save unidiffs. */
    /* safelanes_recalculate(); */
+   return 0;
 }
 
 /**
@@ -228,7 +230,7 @@ void safelanes_destroy( void )
  * signifying "all of them".
  *    @param standing Bit-mask indicating what standing to get.
  *    @param system Star system whose lanes we want.
- *    @return Array (array.h) of matching SafeLane structures. Caller frees.
+ *    @return Array (array.h) of matching `SafeLane` structures. Caller frees.
  */
 SafeLane *safelanes_get( int faction, int standing, const StarSystem *system )
 {
@@ -304,11 +306,15 @@ void safelanes_recalculate( void )
       return;
 
    safelanes_initStacks();
-   safelanes_initOptimizer();
-   for ( int iters_done = 0; safelanes_buildOneTurn( iters_done ) > 0;
-         iters_done++ )
-      ;
-   safelanes_destroyOptimizer();
+   if ( RUNNING_ON_VALGRIND ) {
+      DEBUG( "Running under Valgrind, safelanes not generated!" );
+   } else {
+      safelanes_initOptimizer();
+      for ( int iters_done = 0; safelanes_buildOneTurn( iters_done ) > 0;
+            iters_done++ )
+         ;
+      safelanes_destroyOptimizer();
+   }
    /* Stacks remain available for queries. */
 #if DEBUGGING
    if ( conf.devmode )
@@ -489,7 +495,7 @@ static void safelanes_initStacks_edge( void )
                   break;
                }
             if ( has_approx_midpoint )
-               continue; /* The edge from i to j is disallowed in favor of a
+               continue; /* The edge from i to j is disallowed in favour of a
                             path through k. */
             array_push_back_edge( &edge_stack, j, i );
             array_push_back(
@@ -669,15 +675,15 @@ static void safelanes_initStiff( void )
 }
 
 /**
- * @brief Returns the initial conductivity value (1/length) for edge ei.
+ * @brief Returns the initial conductivity value (1/length) for edge e.
  * The live value is stored in the stiffness matrix; \see safelanes_initStiff
  * above. When a lane is activated, its conductivity is updated to
  * (1+ALPHA)/length.
  */
-static double safelanes_initialConductivity( int ei )
+static double safelanes_initialConductivity( int e )
 {
    double *sv = stiff->x;
-   return lane_faction[ei] ? sv[3 * ei] / ( 1 + ALPHA ) : sv[3 * ei];
+   return lane_faction[e] ? sv[3 * e] / ( 1 + ALPHA ) : sv[3 * e];
 }
 
 /**
@@ -812,7 +818,7 @@ static int safelanes_activateByGradient( const cholmod_dense *Lambda_tilde,
    size_t *lal_bases, lal_base; /**< System si's U and Lambda rows start at
                                    sys_base; its lal rows start at lal_base. */
 
-   lal       = calloc( array_size( faction_stack ), sizeof( cholmod_dense       *) );
+   lal       = calloc( array_size( faction_stack ), sizeof( cholmod_dense * ) );
    lal_bases = calloc( array_size( faction_stack ), sizeof( size_t ) );
    edgeind_opts = array_create( int );
    facind_opts  = array_create_size( int, array_size( faction_stack ) );
