@@ -7,8 +7,8 @@
  * @brief Handles the ship details.
  */
 /** @cond */
-#include "SDL_timer.h"
 #include "physfs.h"
+#include <SDL3/SDL_timer.h>
 
 #include "naev.h"
 /** @endcond */
@@ -25,7 +25,6 @@
 #include "nlua_camera.h"
 #include "nlua_gfx.h"
 #include "nlua_ship.h"
-#include "nstring.h"
 #include "nxml.h"
 #include "opengl_tex.h"
 #include "shipstats.h"
@@ -78,7 +77,7 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
                                       double fw, double fh, double engine_glow,
                                       double t, const glColour *c,
                                       const Lighting *L, const mat4 *H,
-                                      int blit, unsigned int flags );
+                                      int blit );
 
 /**
  * @brief Compares two ship pointers for qsort.
@@ -123,7 +122,7 @@ const Ship *ship_getW( const char *name )
 const char *ship_existsCase( const char *name )
 {
    for ( int i = 0; i < array_size( ship_stack ); i++ )
-      if ( strcasecmp( name, ship_stack[i].name ) == 0 )
+      if ( SDL_strcasecmp( name, ship_stack[i].name ) == 0 )
          return ship_stack[i].name;
    return NULL;
 }
@@ -131,7 +130,7 @@ const char *ship_existsCase( const char *name )
 /**
  * @brief Gets the array (array.h) of all ships.
  */
-const Ship *ship_getAll( void )
+Ship *ship_getAll( void )
 {
    return ship_stack;
 }
@@ -308,17 +307,17 @@ credits_t ship_buyPrice( const Ship *s )
    for ( int i = 0; i < array_size( s->outfit_structure ); i++ ) {
       const Outfit *o = s->outfit_structure[i].data;
       if ( o != NULL )
-         price += o->price;
+         price += outfit_price( o );
    }
    for ( int i = 0; i < array_size( s->outfit_utility ); i++ ) {
       const Outfit *o = s->outfit_utility[i].data;
       if ( o != NULL )
-         price += o->price;
+         price += outfit_price( o );
    }
    for ( int i = 0; i < array_size( s->outfit_weapon ); i++ ) {
       const Outfit *o = s->outfit_weapon[i].data;
       if ( o != NULL )
-         price += o->price;
+         price += outfit_price( o );
    }
 
    return price;
@@ -347,12 +346,12 @@ void ship_renderGfxStore( GLuint fbo, const Ship *s, int size, double dir,
       /* We rotate the model so it's staring at the player and facing slightly
        * down. */
       H = mat4_identity();
-      mat4_rotate( &H, -M_PI_4 + dir, 0.0, 1.0, 0.0 );
-      mat4_rotate( &H, -M_PI_4 * 0.25 + updown, 1.0, 0.0, 0.0 );
+      mat4_rotate( &H, M_PI_4 - dir, 0.0, 1.0, 0.0 );
+      mat4_rotate( &H, M_PI_4 * 0.25 - updown, 1.0, 0.0, 0.0 );
 
       /* Render the model. */
       ship_renderFramebuffer3D( s, fbo, size, gl_screen.nw, gl_screen.nh, glow,
-                                t, &cWhite, &L, &H, 0, OPENGL_TEX_VFLIP );
+                                t, &cWhite, &L, &H, 0 );
       /* Already restore current framebuffer. */
    } else if ( s->gfx_comm != NULL ) {
       glTexture *glcomm;
@@ -360,16 +359,17 @@ void ship_renderGfxStore( GLuint fbo, const Ship *s, int size, double dir,
       int        sx, sy;
 
       glcomm = gl_newImage( s->gfx_comm, 0 );
-      glcomm->flags &= ~OPENGL_TEX_VFLIP;
 
-      scale = MIN( size / glcomm->w, size / glcomm->h );
-      w     = scale * glcomm->w;
-      h     = scale * glcomm->h;
+      scale = MIN( size / tex_w( glcomm ), size / tex_h( glcomm ) );
+      w     = scale * tex_w( glcomm );
+      h     = scale * tex_h( glcomm );
       gl_getSpriteFromDir( &sx, &sy, s->sx, s->sy, dir );
-      tx = glcomm->sw * (double)( sx ) / glcomm->w;
-      ty = glcomm->sh * ( glcomm->sy - (double)sy - 1 ) / glcomm->h;
+      tx = tex_sw( glcomm ) * (double)( sx ) / tex_w( glcomm );
+      ty = tex_sh( glcomm ) * ( tex_sy( glcomm ) - (double)sy - 1 ) /
+           tex_h( glcomm );
       gl_renderTexture( glcomm, ( size - w ) * 0.5, ( size - h ) * 0.5, w, h,
-                        tx, ty, glcomm->srw, glcomm->srh, NULL, 0. );
+                        tx, ty, tex_srw( glcomm ), tex_srh( glcomm ), NULL,
+                        0. );
 
       gl_freeTexture( glcomm );
 
@@ -377,6 +377,12 @@ void ship_renderGfxStore( GLuint fbo, const Ship *s, int size, double dir,
    }
 }
 
+void ship_gfxLoad( Ship *s )
+{
+   ship_setFlag( s, SHIP_NEEDSGFX );
+   ship_gfxLoadNeeded();
+}
+
 /**
  * @brief Get the store gfx.
  */
@@ -405,7 +411,6 @@ glTexture *ship_gfxStore( const Ship *s, int size, double dir, double updown,
 
    snprintf( buf, sizeof( buf ), "%s_fbo_gfx_store_%d", s->name, size );
    gltex = gl_rawTexture( buf, tex, fbosize, fbosize );
-   gltex->flags |= OPENGL_TEX_VFLIP;
 
    return gltex;
 }
@@ -423,6 +428,10 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
    char       buf[STRMAX_SHORT];
    glTexture *gltex;
 
+   // UNUSED
+   (void)tilt;
+   (void)dir;
+
    fbosize = ceil( size / gl_screen.scale );
 
    /* Load graphic if applicable. */
@@ -435,10 +444,9 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 
    if ( s->gfx_3d != NULL ) {
-      Lighting L = ( Lscene == NULL ) ? L_default : *Lscene;
-      mat4     H, Hlight;
-      double   t = SDL_GetTicks() / 1000.;
-      double   rendersize =
+      mat4   H;
+      double t = SDL_GetTicks() / 1000.;
+      double rendersize =
          MIN( MIN( ceil( ship_aa_scale_base * fbosize ), gl_screen.rw ),
               gl_screen.rh );
       snprintf( buf, sizeof( buf ), "%s_fbo_gfx_comm_%d", s->name, size );
@@ -446,17 +454,19 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
       /* We rotate the model so it's staring at the player and facing slightly
        * down. */
       H = mat4_identity();
-      mat4_rotate( &H, -M_PI_4, 0.0, 1.0, 0.0 );
-      mat4_rotate( &H, -M_PI_4 * 0.25, 1.0, 0.0, 0.0 );
+      mat4_rotate( &H, M_PI_4, 0.0, 1.0, 0.0 );
+      mat4_rotate( &H, M_PI_4 * 0.25, 1.0, 0.0, 0.0 );
 
+      /* TODO restore. */
+#if 0
       /* Transform the light so it's consistent with the 3D model. */
-      Hlight = mat4_identity();
+      mat4 Hlight = mat4_identity();
       if ( fabs( tilt ) > DOUBLE_TOL ) {
-         mat4_rotate( &Hlight, M_PI_2, 0.0, 1.0, 0.0 );
-         mat4_rotate( &Hlight, -tilt, 1.0, 0.0, 0.0 );
-         mat4_rotate( &Hlight, dir, 0.0, 1.0, 0.0 );
+         mat4_rotate( &Hlight, -M_PI_2, 0.0, 1.0, 0.0 );
+         mat4_rotate( &Hlight, tilt, 1.0, 0.0, 0.0 );
+         mat4_rotate( &Hlight, -dir, 0.0, 1.0, 0.0 );
       } else
-         mat4_rotate( &Hlight, dir + M_PI_2, 0.0, 1.0, 0.0 );
+         mat4_rotate( &Hlight, -dir - M_PI_2, 0.0, 1.0, 0.0 );
       gltf_lightTransform( &L, &Hlight );
 
       /* Increase ambient a wee bit. */
@@ -464,15 +474,17 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
       L.ambient_g += 0.1;
       L.ambient_b += 0.1;
       L.intensity *= 1.5;
+#endif
 
       /* Render the model. */
       glBindFramebuffer( GL_FRAMEBUFFER, gl_screen.fbo[2] );
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
-      gltf_renderScene( gl_screen.fbo[2], s->gfx_3d, s->gfx_3d->scene_body, &H,
-                        t, rendersize, &L );
+      gltf_renderScene( gl_screen.fbo[2], s->gfx_3d,
+                        gltf_sceneBody( s->gfx_3d ), &H, t, rendersize,
+                        Lscene );
       glBindFramebuffer( GL_READ_FRAMEBUFFER, gl_screen.fbo[2] );
       glBindFramebuffer( GL_DRAW_FRAMEBUFFER, fbo );
-      glBlitFramebuffer( 0, 0, rendersize, rendersize, 0, fbosize, fbosize, 0,
+      glBlitFramebuffer( 0, 0, rendersize, rendersize, 0, 0, fbosize, fbosize,
                          GL_COLOR_BUFFER_BIT, GL_LINEAR );
    } else if ( s->gfx_comm != NULL ) {
       glTexture *glcomm;
@@ -480,11 +492,10 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
 
       snprintf( buf, sizeof( buf ), "%s_fbo_gfx_comm_%d", s->gfx_comm, size );
       glcomm = gl_newImage( s->gfx_comm, 0 );
-      glcomm->flags &= ~OPENGL_TEX_VFLIP;
 
-      scale = MIN( size / glcomm->w, size / glcomm->h );
-      w     = scale * glcomm->w;
-      h     = scale * glcomm->h;
+      scale = MIN( size / tex_w( glcomm ), size / tex_h( glcomm ) );
+      w     = scale * tex_w( glcomm );
+      h     = scale * tex_h( glcomm );
       gl_renderTexture( glcomm, ( size - w ) * 0.5, ( size - h ) * 0.5, w, h,
                         0., 0., 1., 1., NULL, 0. );
 
@@ -499,7 +510,6 @@ glTexture *ship_gfxComm( const Ship *s, int size, double tilt, double dir,
    gl_contextUnset();
 
    gltex = gl_rawTexture( buf, tex, fbosize, fbosize );
-   gltex->flags |= OPENGL_TEX_VFLIP;
 
    return gltex;
 }
@@ -514,7 +524,7 @@ int ship_gfxAnimated( const Ship *s )
 {
    if ( s->gfx_3d == NULL )
       return 0;
-   return ( s->gfx_3d->nanimations > 0 );
+   return ( gltf_numAnimations( s->gfx_3d ) > 0 );
 }
 
 /**
@@ -567,11 +577,14 @@ int ship_size( const Ship *s )
  */
 static int ship_loadSpaceImage( Ship *temp, const char *str, int sx, int sy )
 {
-   unsigned int flags = OPENGL_TEX_MIPMAPS | OPENGL_TEX_VFLIP;
+   unsigned int flags = OPENGL_TEX_MIPMAPS;
    /* If no collision polygon, we use transparency mapping. */
    if ( array_size( temp->polygon.views ) <= 0 )
       flags |= OPENGL_TEX_MAPTRANS;
    temp->gfx_space = gl_newSprite( str, sx, sy, flags );
+   /* 2D graphics will overwrite the size. */
+   temp->size =
+      tex_sw( temp->gfx_space ) * 0.5 + tex_sh( temp->gfx_space ) * 0.5;
    return 0;
 }
 
@@ -597,148 +610,97 @@ int ship_gfxLoaded( const Ship *s )
    return ( ( s->gfx_3d != NULL ) || ( s->gfx_space != NULL ) );
 }
 
-/**
- * @brief Tries to load the graphics for all ships that need it.
- */
-int ship_gfxLoadNeeded( void )
-{
-   ThreadQueue *tq = vpool_create();
-   SDL_GL_MakeCurrent( gl_screen.window, NULL );
-
-   for ( int i = 0; i < array_size( ship_stack ); i++ ) {
-      Ship *s = &ship_stack[i];
-      if ( !ship_isFlag( s, SHIP_NEEDSGFX ) )
-         continue;
-      vpool_enqueue( tq, (int ( * )( void * ))ship_gfxLoad, s );
-      ship_rmFlag( s, SHIP_NEEDSGFX );
-   }
-
-   vpool_wait( tq );
-   vpool_cleanup( tq );
-
-   SDL_GL_MakeCurrent( gl_screen.window, gl_screen.context );
-   return 0;
-}
-
 /**
  * @brief Loads the graphics for a ship if necessary.
  *
  *    @param s Ship to load into.
  */
-int ship_gfxLoad( Ship *s )
+int ship_gfxLoadPost3D( Ship *s )
 {
-   char        str[PATH_MAX], *base;
-   const char *delim, *base_path;
-   const char *ext    = ".webp";
-   const char *buf    = s->gfx_path;
-   int         sx     = s->sx;
-   int         sy     = s->sy;
-   int         engine = !s->noengine;
+   int ntrails = gltf_numTrails( s->gfx_3d );
+   int nmounts = gltf_numMounts( s->gfx_3d );
 
-   /* If already loaded, just ignore. */
-   if ( ship_gfxLoaded( s ) )
-      return 0;
+   /* Replace trails if applicable. */
+   if ( ntrails > 0 ) {
+      array_erase( &s->trail_emitters, array_begin( s->trail_emitters ),
+                   array_end( s->trail_emitters ) );
+      ship_setFlag( s, SHIP_3DTRAILS );
 
-   /* Get base path. */
-   delim     = strchr( buf, '_' );
-   base      = delim == NULL ? strdup( buf ) : strndup( buf, delim - buf );
-   base_path = ( s->base_path != NULL ) ? s->base_path : s->base_type;
+      for ( int i = 0; i < ntrails; i++ ) {
+         const char      *generator = gltf_trailName( s->gfx_3d, i );
+         vec3             pos       = gltf_trailPosition( s->gfx_3d, i );
+         ShipTrailEmitter trail;
 
-   /* Load the 3d model */
-   snprintf( str, sizeof( str ), SHIP_3DGFX_PATH "%s/%s.gltf", base_path, buf );
-   if ( PHYSFS_exists( str ) ) {
-      // DEBUG( "Found 3D graphics for '%s' at '%s'!", s->name, str );
-      s->gfx_3d = gltf_loadFromFile( str );
+         /* New trail. */
+         trail.pos = pos;
+         vec3_scale( &trail.pos, s->size * 0.5 ); /* Convert to "pixels" */
+         trail.trail_spec = trailSpec_get( generator );
+         trail.flags      = 0;
 
-      /* Replace trails if applicable. */
-      if ( array_size( s->gfx_3d->trails ) > 0 ) {
-         array_erase( &s->trail_emitters, array_begin( s->trail_emitters ),
-                      array_end( s->trail_emitters ) );
-         ship_setFlag( s, SHIP_3DTRAILS );
-
-         for ( int i = 0; i < array_size( s->gfx_3d->trails ); i++ ) {
-            GltfTrail       *t = &s->gfx_3d->trails[i];
-            ShipTrailEmitter trail;
-
-            /* New trail. */
-            trail.pos = t->pos;
-            vec3_scale( &trail.pos, s->size * 0.5 ); /* Convert to "pixels" */
-            trail.trail_spec = trailSpec_get( t->generator );
-            trail.flags      = 0;
-
-            if ( trail.trail_spec != NULL )
-               array_push_back( &s->trail_emitters, trail );
-         }
-      }
-
-      /* Replace mount points if applicable. */
-      if ( array_size( s->gfx_3d->mounts ) > 0 ) {
-         int n = array_size( s->gfx_3d->mounts );
-
-         ship_setFlag( s, SHIP_3DMOUNTS );
-         if ( n < array_size( s->outfit_weapon ) )
-            WARN( _( "Number of 3D weapon mounts from GLTF file for ship '%s' "
-                     "is less than the "
-                     "number of ship weapons! Got %d, expected at least %d." ),
-                  s->name, n, array_size( s->outfit_weapon ) );
-
-         for ( int i = 0; i < array_size( s->outfit_weapon ); i++ ) {
-            ShipMount *sm = &s->outfit_weapon[i].mount;
-            vec3_copy( &sm->pos,
-                       &s->gfx_3d->mounts[i % n].pos ); /* Loop over. */
-            vec3_scale( &sm->pos, s->size * 0.5 ); /* Convert to "pixels" */
-         }
+         if ( trail.trail_spec != NULL )
+            array_push_back( &s->trail_emitters, trail );
       }
    }
 
-   /* Determine extension path. */
-   if ( buf[0] == '/' ) /* absolute path. */
-      snprintf( str, sizeof( str ), "%s", buf );
-   else {
-      snprintf( str, sizeof( str ), SHIP_GFX_PATH "%s/%s%s", base, buf, ext );
-      if ( !PHYSFS_exists( str ) ) {
-         ext = ".png";
-         snprintf( str, sizeof( str ), SHIP_GFX_PATH "%s/%s%s", base, buf,
-                   ext );
+   /* Replace mount points if applicable. */
+   if ( nmounts > 0 ) {
+      ship_setFlag( s, SHIP_3DMOUNTS );
+      if ( nmounts < array_size( s->outfit_weapon ) )
+         WARN( _( "Number of 3D weapon mounts from GLTF file for ship '%s' "
+                  "is less than the "
+                  "number of ship weapons! Got %d, expected at least %d." ),
+               s->name, nmounts, array_size( s->outfit_weapon ) );
+
+      for ( int i = 0; i < array_size( s->outfit_weapon ); i++ ) {
+         vec3 pos = gltf_mountPosition( s->gfx_3d, i % nmounts );
+
+         ShipMount *sm = &s->outfit_weapon[i].mount;
+         vec3_copy( &sm->pos, &pos );           /* Loop over. */
+         vec3_scale( &sm->pos, s->size * 0.5 ); /* Convert to "pixels" */
       }
    }
 
    /* Load the polygon. */
    ship_loadPLG( s,
                  ( s->polygon_path != NULL ) ? s->polygon_path : s->gfx_path );
+   return 0;
+}
 
-   /* If we have 3D and polygons, we'll ignore the 2D stuff. */
-   if ( ( s->gfx_3d != NULL ) && ( array_size( s->polygon.views ) > 0 ) ) {
-      free( base );
-      return 0;
-   }
+int ship_gfxLoad2D( Ship *s, const char *base, const char *ext )
+{
+   int  sx     = s->sx;
+   int  sy     = s->sy;
+   int  engine = !s->noengine;
+   char str[PATH_MAX];
+
+   /* Load the polygon. */
+   ship_loadPLG( s,
+                 ( s->polygon_path != NULL ) ? s->polygon_path : s->gfx_path );
 
    /* Get the comm graphic for future loading. */
    if ( s->gfx_comm == NULL )
-      SDL_asprintf( &s->gfx_comm, SHIP_GFX_PATH "%s/%s" SHIP_COMM "%s", base,
-                    buf, ext );
+      SDL_asprintf( &s->gfx_comm, "%s" SHIP_COMM "%s", base, ext );
 
    /* Load the space sprite. */
+   snprintf( str, sizeof( str ), "%s%s", base, ext );
    ship_loadSpaceImage( s, str, sx, sy );
 
    /* Load the engine sprite .*/
    if ( engine ) {
-      snprintf( str, sizeof( str ), SHIP_GFX_PATH "%s/%s" SHIP_ENGINE "%s",
-                base, buf, ext );
+      snprintf( str, sizeof( str ), "%s" SHIP_ENGINE "%s", base, ext );
       ship_loadEngineImage( s, str, sx, sy );
       if ( s->gfx_engine == NULL )
          WARN( _( "Ship '%s' does not have an engine sprite (%s)." ), s->name,
                str );
    }
-   free( base );
 
 #if 0
 #if DEBUGGING
    if ( ( s->gfx_space != NULL ) &&
-        ( round( s->size ) != round( s->gfx_space->sw ) ) )
+        ( round( s->size ) != round( tex_sw(s->gfx_space)) ) )
       WARN( ( "Mismatch between 'size' and 'gfx_space' sprite size for ship "
               "'%s'! 'size' should be %.0f!" ),
-            s->name, s->gfx_space->sw );
+            s->name, tex_sw(s->gfx_space));
 #endif /* DEBUGGING */
 #endif
 
@@ -760,7 +722,7 @@ static int ship_loadPLG( Ship *temp, const char *buf )
    if ( temp->gfx_3d != NULL )
       snprintf( file, sizeof( file ), "%s%s.xml", SHIP_POLYGON_PATH3D, buf );
    else
-      snprintf( file, sizeof( file ), "%s%s.xml", SHIP_POLYGON_PATH, buf );
+      snprintf( file, sizeof( file ), "%s%s.xml", SHIP_POLYGON_PATH2D, buf );
 
    /* See if the file does exist. */
    if ( !PHYSFS_exists( file ) ) {
@@ -874,6 +836,12 @@ static int ship_parseSlot( Ship *temp, ShipOutfitSlot *slot,
    slot->slot.size = base_size;
    slot->slot.type = type;
 
+   /* Make sure default outfit fits slot. */
+   if ( ( slot->data != NULL ) && !outfit_fitsSlot( slot->data, &slot->slot ) )
+      WARN( _( "Ship '%s' has default outfit '%s' which does not fit in the "
+               "slot it is equipped by default." ),
+            temp->name, outfit_name( slot->data ) );
+
    /* Required slots need a default outfit. */
    if ( slot->required && ( slot->data == NULL ) )
       WARN( _( "Ship '%s' has required slot without a default outfit." ),
@@ -882,6 +850,51 @@ static int ship_parseSlot( Ship *temp, ShipOutfitSlot *slot,
    return 0;
 }
 
+static int checkSlotNameDuplicates( const Ship           *temp,
+                                    const ShipOutfitSlot *slot )
+{
+   int ret = 0;
+   if ( slot->name == NULL )
+      return 0;
+   for ( int i = 0; i < array_size( temp->outfit_structure ); i++ ) {
+      const ShipOutfitSlot *so = &temp->outfit_structure[i];
+      if ( slot == so )
+         continue;
+      if ( so->name == NULL )
+         continue;
+      if ( strcmp( so->name, slot->name ) != 0 )
+         continue;
+      WARN( _( "Ship '%s' has duplicated slot name '%s'!" ), temp->name,
+            slot->name );
+      ret = 1;
+   }
+   for ( int i = 0; i < array_size( temp->outfit_utility ); i++ ) {
+      const ShipOutfitSlot *so = &temp->outfit_utility[i];
+      if ( slot == so )
+         continue;
+      if ( so->name == NULL )
+         continue;
+      if ( strcmp( so->name, slot->name ) != 0 )
+         continue;
+      WARN( _( "Ship '%s' has duplicated slot name '%s'!" ), temp->name,
+            slot->name );
+      ret = 1;
+   }
+   for ( int i = 0; i < array_size( temp->outfit_weapon ); i++ ) {
+      const ShipOutfitSlot *so = &temp->outfit_weapon[i];
+      if ( slot == so )
+         continue;
+      if ( so->name == NULL )
+         continue;
+      if ( strcmp( so->name, slot->name ) != 0 )
+         continue;
+      WARN( _( "Ship '%s' has duplicated slot name '%s'!" ), temp->name,
+            slot->name );
+      ret = 1;
+   }
+   return ret;
+}
+
 /**
  * @brief Extracts the in-game ship from an XML node.
  *
@@ -923,7 +936,7 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       temp->trail_emitters = array_create( ShipTrailEmitter );
 
       /* Lua defaults. */
-      temp->lua_env            = LUA_NOREF;
+      temp->lua_env            = NULL;
       temp->lua_descextra      = LUA_NOREF;
       temp->lua_init           = LUA_NOREF;
       temp->lua_cleanup        = LUA_NOREF;
@@ -931,6 +944,8 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       temp->lua_dt             = 0.1;
       temp->lua_explode_init   = LUA_NOREF;
       temp->lua_explode_update = LUA_NOREF;
+      temp->lua_onshootany     = LUA_NOREF;
+      temp->lua_cooldown       = LUA_NOREF;
 
       /* Get name. */
       xmlr_attr_strd_free( parent, "name", temp->name );
@@ -950,8 +965,12 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
 #define STRDUP_( x ) ( ( x == NULL ) ? NULL : strdup( x ) )
 #define ARRAYDUP_( x, y )                                                      \
    do {                                                                        \
-      for ( int i = 0; i < array_size( y ); i++ )                              \
-         array_push_back( &x, y[i] );                                          \
+      for ( int i = 0; i < array_size( y ); i++ ) {                            \
+         ShipOutfitSlot sos = y[i];                                            \
+         if ( sos.name != NULL )                                               \
+            sos.name = strdup( sos.name );                                     \
+         array_push_back( &x, sos );                                           \
+      }                                                                        \
    } while ( 0 )
       Ship  t             = *temp;
       Ship *base          = (Ship *)ship_get( temp->inherits );
@@ -969,6 +988,7 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       temp->description   = STRDUP_( base->description );
       temp->desc_extra    = STRDUP_( base->desc_extra );
       temp->gfx_path      = STRDUP_( base->gfx_path );
+      temp->gfx_extension = STRDUP_( base->gfx_extension );
       temp->polygon_path  = STRDUP_( base->polygon_path );
       temp->gfx_comm      = STRDUP_( base->gfx_comm );
       if ( base->gfx_overlays != NULL ) {
@@ -986,15 +1006,8 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       ARRAYDUP_( temp->outfit_weapon, base->outfit_weapon );
       temp->desc_stats = STRDUP_( base->desc_stats );
       temp->stats      = NULL;
-      ShipStatList *ll = base->stats;
-      while ( ll != NULL ) {
-         ShipStatList *ln = calloc( 1, sizeof( ShipStatList ) );
-         *ln              = *ll;
-         ln->next         = temp->stats;
-         temp->stats      = ln;
-         ll               = ll->next;
-      }
-      temp->tags = array_create( char * );
+      temp->stats      = ss_dupList( base->stats );
+      temp->tags       = array_create( char * );
       for ( int i = 0; i < array_size( base->tags ); i++ )
          array_push_back( &temp->tags, strdup( base->tags[i] ) );
       temp->lua_file = STRDUP_( base->lua_file );
@@ -1017,11 +1030,8 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
          temp->class = ship_classFromString( xml_get( node ) );
          continue;
       }
-      if ( xml_isNode( node, "gfx" ) || xml_isNode( node, "GFX" ) ) {
+      if ( xml_isNode( node, "gfx" ) ) {
          const char *str = xml_get( node );
-         /* TODO remove for 0.13.0 */
-         if ( xml_isNode( node, "GFX" ) )
-            WARN( _( "Ship '%s': using <GFX> instead of <gfx>!" ), temp->name );
 
          /* Get base graphic name. */
          if ( str == NULL ) {
@@ -1039,6 +1049,7 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
          xmlr_attr_strd_free( node, "comm", temp->gfx_comm );
          xmlr_attr_int( node, "noengine", temp->noengine );
          xmlr_attr_strd_free( node, "polygon", temp->polygon_path );
+         xmlr_attr_strd_free( node, "extension", temp->gfx_extension );
 
          /* Graphics are now lazy loaded. */
 
@@ -1078,7 +1089,7 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       }
       xmlr_float( node, "time_mod", temp->dt_default );
       xmlr_long( node, "price", temp->price );
-      xmlr_strd_free( node, "license", temp->license );
+      xmlr_strd_free( node, "licence", temp->license );
       xmlr_strd_free( node, "cond", temp->cond );
       xmlr_strd_free( node, "condstr", temp->condstr );
       xmlr_strd_free( node, "fabricator", temp->fabricator );
@@ -1190,8 +1201,14 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
       }
       if ( xml_isNode( node, "slots" ) ) {
          /* Clean up, just in case. */
+         for ( int i = 0; i < array_size( temp->outfit_structure ); i++ )
+            free( temp->outfit_structure[i].name );
          array_free( temp->outfit_structure );
+         for ( int i = 0; i < array_size( temp->outfit_utility ); i++ )
+            free( temp->outfit_utility[i].name );
          array_free( temp->outfit_utility );
+         for ( int i = 0; i < array_size( temp->outfit_weapon ); i++ )
+            free( temp->outfit_weapon[i].name );
          array_free( temp->outfit_weapon );
          /* Allocate the space. */
          temp->outfit_structure = array_create( ShipOutfitSlot );
@@ -1212,6 +1229,11 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
                ship_parseSlot( temp, &array_grow( &temp->outfit_weapon ),
                                OUTFIT_SLOT_WEAPON, cur );
             else if ( xml_isNode( cur, "intrinsic" ) ) {
+               /* Deprecated remove in 0.14.0 */
+               WARN(
+                  _( "Ship '%s' using deprecated api for intrinsic outfits. "
+                     "Please define them in <intrinsics> instead of <slots>." ),
+                  temp->name );
                const Outfit *o = outfit_get( xml_get( cur ) );
                if ( o == NULL ) {
                   WARN( _( "Ship '%s' has unknown intrinsic outfit '%s'" ),
@@ -1231,32 +1253,42 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
          array_shrink( &temp->outfit_weapon );
          continue;
       }
+      if ( xml_isNode( node, "intrinsics" ) ) {
+         /* Have to reset in case of inheriting ship. */
+         array_free( temp->outfit_intrinsic );
+         temp->outfit_intrinsic = (Outfit const **)array_create( Outfit * );
+
+         xmlNodePtr cur = node->children;
+         do {
+            xml_onlyNodes( cur );
+            if ( xml_isNode( cur, "intrinsic" ) ) {
+               const Outfit *o = outfit_get( xml_get( cur ) );
+               if ( o == NULL ) {
+                  WARN( _( "Ship '%s' has unknown intrinsic outfit '%s'" ),
+                        temp->name, xml_get( cur ) );
+                  continue;
+               }
+               array_push_back( &temp->outfit_intrinsic, o );
+            } else
+               WARN( _( "Ship '%s' has unknown intrinsic node '%s'." ),
+                     temp->name, cur->name );
+         } while ( xml_nextNode( cur ) );
+         continue;
+      }
 
       /* Parse ship stats. */
       if ( xml_isNode( node, "stats" ) ) {
-         xmlNodePtr cur = node->children;
          /* Clear if duplicated. */
          if ( temp->stats != NULL ) {
             ss_free( temp->stats );
             temp->stats = NULL;
          }
-         do {
-            ShipStatList *ll;
-            xml_onlyNodes( cur );
-            ll = ss_listFromXML( cur );
-            if ( ll != NULL ) {
-               ll->next    = temp->stats;
-               temp->stats = ll;
-               continue;
-            }
-            WARN( _( "Ship '%s' has unknown stat '%s'." ), temp->name,
-                  cur->name );
-         } while ( xml_nextNode( cur ) );
+         temp->stats = ss_listFromXML( node );
 
          /* Load array. */
          ss_sort( &temp->stats );
          ss_statsInit( &temp->stats_array );
-         ss_statsMergeFromList( &temp->stats_array, temp->stats );
+         ss_statsMergeFromList( &temp->stats_array, temp->stats, 0 );
 
          /* Create description. */
          if ( temp->stats != NULL ) {
@@ -1306,12 +1338,20 @@ static int ship_parse( Ship *temp, const char *filename, int firstpass )
    temp->dmg_absorb /= 100.;
    temp->turn *= M_PI / 180.; /* Convert to rad. */
 
-   /* Check license. */
+   /* Check licence. */
    if ( temp->license && !outfit_licenseExists( temp->license ) )
-      WARN( _( "Ship '%s' has inexistent license requirement '%s'!" ),
+      WARN( _( "Ship '%s' has inexistent licence requirement '%s'!" ),
             temp->name, temp->license );
 
-      /* Ship XML validator */
+   /* Make sure no duplicate slot names. */
+   for ( int i = 0; i < array_size( temp->outfit_structure ); i++ )
+      checkSlotNameDuplicates( temp, &temp->outfit_structure[i] );
+   for ( int i = 0; i < array_size( temp->outfit_utility ); i++ )
+      checkSlotNameDuplicates( temp, &temp->outfit_utility[i] );
+   for ( int i = 0; i < array_size( temp->outfit_weapon ); i++ )
+      checkSlotNameDuplicates( temp, &temp->outfit_weapon[i] );
+
+   /* Ship XML validator */
 #define MELEMENT( o, s )                                                       \
    if ( o )                                                                    \
    WARN( _( "Ship '%s' missing '%s' element" ), temp->name, s )
@@ -1355,7 +1395,7 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
                                       double fw, double fh, double engine_glow,
                                       double t, const glColour *c,
                                       const Lighting *L, const mat4 *H,
-                                      int blit, unsigned int flags )
+                                      int blit )
 {
    double      scale = ship_aa_scale * size;
    GltfObject *obj   = s->gfx_3d;
@@ -1378,10 +1418,11 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
    mat4_scale_xy( &tex_mat, scale / ship_fbos, scale / ship_fbos );
 
    /* Actually render. */
-   if ( ( engine_glow > 0. ) && ( obj->scene_engine >= 0 ) ) {
+   int scene_body   = gltf_sceneBody( obj );
+   int scene_engine = gltf_sceneEngine( obj );
+   if ( ( engine_glow > 0. ) && ( scene_engine >= 0 ) ) {
       if ( engine_glow >= 1. ) {
-         gltf_renderScene( ship_fbo[0], obj, obj->scene_engine, H, t, scale,
-                           L );
+         gltf_renderScene( ship_fbo[0], obj, scene_engine, H, t, scale, L );
       } else {
          /* More scissors on the remaining ship fbos. */
          glEnable( GL_SCISSOR_TEST );
@@ -1392,14 +1433,14 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
          glDisable( GL_SCISSOR_TEST );
 
          /* First render separately. */
-         gltf_renderScene( ship_fbo[1], obj, obj->scene_body, H, t, scale, L );
-         gltf_renderScene( ship_fbo[2], obj, obj->scene_engine, H, t, scale,
-                           L );
+         gltf_renderScene( ship_fbo[1], obj, scene_body, H, t, scale, L );
+         gltf_renderScene( ship_fbo[2], obj, scene_engine, H, t, scale, L );
 
          /* Now merge to main framebuffer. */
          glBindFramebuffer( GL_FRAMEBUFFER, ship_fbo[0] );
-         gl_renderTextureInterpolateRawH( ship_tex[2], ship_tex[1], engine_glow,
-                                          &projection, &tex_mat, &cWhite );
+         gl_renderTextureInterpolateRawH( ship_tex[2], ship_tex[1], 0, 0,
+                                          engine_glow, &projection, &tex_mat,
+                                          &cWhite );
 
          /* Copy depth over. */
          GLint blitsize = scale;
@@ -1412,7 +1453,7 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
           * framebuffer in the gui. */
       }
    } else
-      gltf_renderScene( ship_fbo[0], obj, obj->scene_body, H, t, scale, L );
+      gltf_renderScene( ship_fbo[0], obj, scene_body, H, t, scale, L );
 
    /*
     * First do sharpen pass.
@@ -1453,28 +1494,15 @@ static void ship_renderFramebuffer3D( const Ship *s, GLuint fbo, double size,
       GLint sout = ceil( size / gl_screen.scale ) + 1;
       glBindFramebuffer( GL_READ_FRAMEBUFFER, ship_fbo[1] );
       glBindFramebuffer( GL_DRAW_FRAMEBUFFER, fbo );
-      if ( flags & OPENGL_TEX_VFLIP ) {
-         glBlitFramebuffer( 0, 0, sin, sin, 0, sout, sout, 0,
-                            GL_COLOR_BUFFER_BIT, GL_LINEAR );
-         /* Depth can only use NEAREST filtering. */
-         glBindFramebuffer( GL_READ_FRAMEBUFFER, ship_fbo[0] );
-         glBlitFramebuffer( 0, 0, sin, sin, 0, sout, sout, 0,
-                            GL_DEPTH_BUFFER_BIT, GL_NEAREST );
-      } else {
-         glBlitFramebuffer( 0, 0, sin, sin, 0, 0, sout, sout,
-                            GL_COLOR_BUFFER_BIT, GL_LINEAR );
-         glBindFramebuffer( GL_READ_FRAMEBUFFER, ship_fbo[0] );
-         glBlitFramebuffer( 0, 0, sin, sin, 0, 0, sout, sout,
-                            GL_DEPTH_BUFFER_BIT, GL_NEAREST );
-      }
+      glBlitFramebuffer( 0, 0, sin, sin, 0, 0, sout, sout, GL_COLOR_BUFFER_BIT,
+                         GL_LINEAR );
+      glBindFramebuffer( GL_READ_FRAMEBUFFER, ship_fbo[0] );
+      glBlitFramebuffer( 0, 0, sin, sin, 0, 0, sout, sout, GL_DEPTH_BUFFER_BIT,
+                         GL_NEAREST );
    } else {
       glBindFramebuffer( GL_FRAMEBUFFER, fbo );
       tex_mat = mat4_identity();
-      if ( flags & OPENGL_TEX_VFLIP ) {
-         tex_mat.m[1][1] = -1.;
-         tex_mat.m[3][1] = 0.5;
-      }
-      gl_renderTextureRawH( ship_tex[1], &projection, &tex_mat, &cWhite );
+      gl_renderTextureRawH( ship_tex[1], 0, &projection, &tex_mat, &cWhite );
    }
 
    glBindFramebuffer( GL_FRAMEBUFFER, gl_screen.current_fbo );
@@ -1500,14 +1528,14 @@ void ship_renderFramebuffer( const Ship *s, GLuint fbo, double fw, double fh,
       /* Determine the model transformation. */
       mat4 H = mat4_identity();
       if ( fabs( tilt ) > DOUBLE_TOL ) {
-         mat4_rotate( &H, M_PI_2, 0.0, 1.0, 0.0 );
-         mat4_rotate( &H, -tilt, 1.0, 0.0, 0.0 );
-         mat4_rotate( &H, dir, 0.0, 1.0, 0.0 );
+         mat4_rotate( &H, -M_PI_2, 0.0, 1.0, 0.0 );
+         mat4_rotate( &H, tilt, 1.0, 0.0, 0.0 );
+         mat4_rotate( &H, -dir, 0.0, 1.0, 0.0 );
       } else
-         mat4_rotate( &H, dir + M_PI_2, 0.0, 1.0, 0.0 );
+         mat4_rotate( &H, -dir - M_PI_2, 0.0, 1.0, 0.0 );
 
       ship_renderFramebuffer3D( s, fbo, s->size, fw, fh, engine_glow, t, c, L,
-                                &H, 1, 0 );
+                                &H, 1 );
       /* Already restore current framebuffer. */
    } else {
       double           tx, ty;
@@ -1521,20 +1549,21 @@ void ship_renderFramebuffer( const Ship *s, GLuint fbo, double fw, double fh,
 
       /* Only clear the necessary area. */
       glEnable( GL_SCISSOR_TEST );
-      glScissor( 0, 0, sa->sw / gl_screen.scale + 1,
-                 sa->sh / gl_screen.scale + 1 );
+      glScissor( 0, 0, tex_sw( sa ) / gl_screen.scale + 1,
+                 tex_sh( sa ) / gl_screen.scale + 1 );
       glClear( GL_COLOR_BUFFER_BIT );
       glDisable( GL_SCISSOR_TEST );
 
       /* Texture coords */
-      tx = sa->sw * (double)( sx ) / sa->w;
-      ty = sa->sh * ( sa->sy - (double)sy - 1 ) / sa->h;
+      tx = tex_sw( sa ) * (double)( sx ) / tex_w( sa );
+      ty = tex_sh( sa ) * ( tex_sy( sa ) - (double)sy - 1 ) / tex_h( sa );
 
       tmpm           = gl_view_matrix;
       gl_view_matrix = mat4_ortho( 0., fw, 0, fh, -1., 1. );
 
-      gl_renderTextureInterpolate( sb, sa, engine_glow, 0., 0., sa->sw, sa->sh,
-                                   tx, ty, sa->srw, sa->srh, c );
+      gl_renderTextureInterpolate( sb, sa, engine_glow, 0., 0., tex_sw( sa ),
+                                   tex_sh( sa ), tx, ty, tex_srw( sa ),
+                                   tex_srh( sa ), c );
 
       gl_view_matrix = tmpm;
 
@@ -1571,7 +1600,6 @@ int ships_load( void )
 #if DEBUGGING
    Uint32 time = SDL_GetTicks();
 #endif /* DEBUGGING */
-   ThreadQueue    *tq       = vpool_create();
    ShipThreadData *shipdata = array_create( ShipThreadData );
 
    /* Validity. */
@@ -1595,6 +1623,8 @@ int ships_load( void )
    }
    array_free( ship_files );
 
+#if 1
+   ThreadQueue *tq = vpool_create();
    /* Enqueue the jobs after the data array is done. */
    SDL_GL_MakeCurrent( gl_screen.window, NULL );
    for ( int i = 0; i < array_size( shipdata ); i++ )
@@ -1603,6 +1633,10 @@ int ships_load( void )
    vpool_wait( tq );
    vpool_cleanup( tq );
    SDL_GL_MakeCurrent( gl_screen.window, gl_screen.context );
+#else
+   for ( int i = 0; i < array_size( shipdata ); i++ )
+      ship_parseThread( &shipdata[i] );
+#endif
 
    /* Properly load the data. */
    for ( int i = 0; i < array_size( shipdata ); i++ ) {
@@ -1642,9 +1676,9 @@ int ships_load( void )
       if ( s->lua_file == NULL )
          continue;
 
-      nlua_env env;
-      size_t   sz;
-      char    *dat = ndata_read( s->lua_file, &sz );
+      nlua_env *env;
+      size_t    sz;
+      char     *dat = ndata_read( s->lua_file, &sz );
       if ( dat == NULL ) {
          WARN( _( "Ship '%s' failed to read Lua '%s'!" ), s->name,
                s->lua_file );
@@ -1665,7 +1699,7 @@ int ships_load( void )
          lua_pop( naevL, 1 );
          nlua_freeEnv( s->lua_env );
          free( dat );
-         s->lua_env = LUA_NOREF;
+         s->lua_env = NULL;
          continue;
       }
       free( dat );
@@ -1683,6 +1717,8 @@ int ships_load( void )
          nlua_refenvtype( env, "explode_init", LUA_TFUNCTION );
       s->lua_explode_update =
          nlua_refenvtype( env, "explode_update", LUA_TFUNCTION );
+      s->lua_onshootany = nlua_refenvtype( env, "onshootany", LUA_TFUNCTION );
+      s->lua_cooldown   = nlua_refenvtype( env, "cooldown", LUA_TFUNCTION );
 
       /* We're just going to run the script and paste here for now. */
       if ( s->lua_descextra != LUA_NOREF ) {
@@ -1733,6 +1769,17 @@ void ships_resize( void )
    for ( int i = 0; i < SHIP_FBO; i++ ) {
       gl_fboCreate( &ship_fbo[i], &ship_tex[i], ship_fbos, ship_fbos );
       gl_fboAddDepth( ship_fbo[i], &ship_texd[i], ship_fbos, ship_fbos );
+
+      /* Names for debugging. */
+      if ( gl_supportsDebug() ) {
+         char buf[STRMAX_SHORT];
+         snprintf( buf, sizeof( buf ), "Ship Framebuffer %d", i );
+         glObjectLabel( GL_FRAMEBUFFER, ship_fbo[i], strlen( buf ), buf );
+         snprintf( buf, sizeof( buf ), "Ship Texture %d", i );
+         glObjectLabel( GL_TEXTURE, ship_tex[i], strlen( buf ), buf );
+         snprintf( buf, sizeof( buf ), "Ship Depth %d", i );
+         glObjectLabel( GL_TEXTURE, ship_texd[i], strlen( buf ), buf );
+      }
    }
    gl_checkErr();
 }
@@ -1806,7 +1853,7 @@ void ships_free( void )
 
       /* Free Lua. */
       nlua_freeEnv( s->lua_env );
-      s->lua_env = LUA_NOREF;
+      s->lua_env = NULL;
       free( s->lua_file );
    }
 
