@@ -7,8 +7,8 @@
  * @brief Handles the special effects.
  */
 /** @cond */
-#include "SDL_haptic.h"
-#include "SDL_timer.h"
+#include <SDL3/SDL_haptic.h>
+#include <SDL3/SDL_timer.h>
 /** @endcond */
 
 #include "spfx.h"
@@ -47,8 +47,8 @@
 /* Trail stuff. */
 #define TRAIL_UPDATE_DT                                                        \
    0.05 /**< Rate (in seconds) at which trail is updated. */
-static TrailSpec   *trail_spec_stack; /**< Trail specifications. */
-static Trail_spfx **trail_spfx_stack; /**< Active trail effects. */
+static TrailSpec   *trail_spec_stack = NULL; /**< Trail specifications. */
+static Trail_spfx **trail_spfx_stack = NULL; /**< Active trail effects. */
 
 /*
  * Special hard-coded special effects
@@ -469,8 +469,14 @@ void spfx_free( void )
    noise_delete( shake_noise );
 
    /* Free the trails. */
-   for ( int i = 0; i < array_size( trail_spfx_stack ); i++ )
-      spfx_trail_free( trail_spfx_stack[i] );
+   for ( int i = 0; i < array_size( trail_spfx_stack ); i++ ) {
+      /* Not sure what is going on here, but it seems like the refcount getting
+       * wrong and it is failing the assert on some systems. */
+      // spfx_trail_free( trail_spfx_stack[i] );
+      Trail_spfx *trail = trail_spfx_stack[i];
+      free( trail->point_ringbuf );
+      free( trail );
+   }
    array_free( trail_spfx_stack );
    trail_spfx_stack = NULL;
 
@@ -488,7 +494,7 @@ void spfx_free( void )
    trail_spec_stack = NULL;
 
    /* Get rid of Lua effects. */
-   spfxL_exit();
+   // spfxL_exit();
 }
 
 /**
@@ -727,7 +733,7 @@ static void spfx_updateDamage( double dt )
 }
 
 /**
- * @brief Initalizes a trail.
+ * @brief Initializes a trail.
  *
  *    @return Pointer to initialized trail. When done (due e.g. to pilot death),
  * call spfx_trail_remove.
@@ -758,16 +764,16 @@ Trail_spfx *spfx_trail_create( const TrailSpec *spec )
 void spfx_update_trails( double dt )
 {
    int n = array_size( trail_spfx_stack );
-   for ( int i = 0; i < n; i++ ) {
+   for ( int i = n - 1; i >= 0; i-- ) {
       Trail_spfx *trail = trail_spfx_stack[i];
-      spfx_trail_update( trail, dt );
       if ( !trail->refcount && !trail_size( trail ) ) {
          spfx_trail_free( trail );
-         trail_spfx_stack[i--] = trail_spfx_stack[--n];
+         array_erase( &trail_spfx_stack, &trail_spfx_stack[i],
+                      &trail_spfx_stack[i + 1] );
+      } else {
+         spfx_trail_update( trail, dt );
       }
    }
-   if ( n < array_size( trail_spfx_stack ) )
-      array_resize( &trail_spfx_stack, n );
 }
 
 /**
@@ -780,7 +786,8 @@ static void spfx_trail_update( Trail_spfx *trail, double dt )
 {
    GLfloat rel_dt = dt / trail->spec->ttl;
    /* Remove outdated elements. */
-   while ( trail->iread < trail->iwrite && trail_front( trail ).t < rel_dt )
+   while ( trail->iread < trail->iwrite &&
+           trail_front( trail ).t < -TRAIL_UPDATE_DT )
       trail->iread++;
 
    /* Update the other trail point's properties. */
@@ -789,6 +796,12 @@ static void spfx_trail_update( Trail_spfx *trail, double dt )
       double      mod         = dt * trail_point->t * trail->spec->accel_mod;
       trail_point->x += trail_point->dx * mod;
       trail_point->y += trail_point->dy * mod;
+      if ( ( trail_point->mode != MODE_IDLE ) &&
+           ( trail->spec->accel_base > 0. ) ) {
+         mod = dt * trail_point->t / ( trail_point->dx + trail_point->dy );
+         trail_point->x += trail->spec->accel_base * mod * trail_point->dx;
+         trail_point->y += trail->spec->accel_base * mod * trail_point->dy;
+      }
       trail_point->t -= rel_dt;
    }
 
@@ -811,11 +824,10 @@ static void spfx_trail_update( Trail_spfx *trail, double dt )
 void spfx_trail_sample( Trail_spfx *trail, double x, double y, double z,
                         double dx, double dy, TrailMode mode, int force )
 {
-   TrailPoint p;
-
    if ( !force && trail->spec->style[mode].col.a <= 0. )
       return;
 
+   TrailPoint p;
    p.x    = x;
    p.y    = y;
    p.z    = z;
@@ -1032,7 +1044,7 @@ static int spfx_hapticInit( void )
    efx->periodic.fade_length    = 1000;
    efx->periodic.fade_level     = 0;
 
-   haptic_rumble = SDL_HapticNewEffect( haptic, efx );
+   haptic_rumble = SDL_CreateHapticEffect( haptic, efx );
    if ( haptic_rumble < 0 ) {
       WARN( _( "Unable to upload haptic effect: %s." ), SDL_GetError() );
       return -1;
@@ -1061,7 +1073,7 @@ static void spfx_hapticRumble( double mod )
       return;
 
    /* Stop the effect if it was playing. */
-   SDL_HapticStopEffect( haptic, haptic_rumble );
+   SDL_StopHapticEffect( haptic, haptic_rumble );
 
    /* Get length and magnitude. */
    len = 1000. * shake_force_mod / SPFX_SHAKE_DECAY;
@@ -1072,14 +1084,14 @@ static void spfx_hapticRumble( double mod )
    efx->periodic.magnitude   = (int16_t)mag;
    efx->periodic.length      = (uint32_t)len;
    efx->periodic.fade_length = MIN( efx->periodic.length, 1000 );
-   if ( SDL_HapticUpdateEffect( haptic, haptic_rumble, &haptic_rumbleEffect ) <
+   if ( SDL_UpdateHapticEffect( haptic, haptic_rumble, &haptic_rumbleEffect ) ==
         0 ) {
       WARN( _( "Failed to update haptic effect: %s." ), SDL_GetError() );
       return;
    }
 
    /* Run the new effect. */
-   SDL_HapticRunEffect( haptic, haptic_rumble, 1 );
+   SDL_RunHapticEffect( haptic, haptic_rumble, 1 );
 
    /* Set timer again. */
    haptic_lastUpdate += HAPTIC_UPDATE_INTERVAL;
@@ -1152,8 +1164,8 @@ static void spfx_renderStack( SPFX *spfx_stack )
          int sx, sy;
 
          /* Simplifies */
-         sx = (int)effect->gfx->sx;
-         sy = (int)effect->gfx->sy;
+         sx = (int)tex_sx( effect->gfx );
+         sy = (int)tex_sy( effect->gfx );
 
          if ( !paused ) { /* don't calculate frame if paused */
             double time =
@@ -1178,6 +1190,7 @@ static void spfx_renderStack( SPFX *spfx_stack )
 void spfx_render( int layer, double dt )
 {
    NTracingZone( _ctx, 1 );
+   gl_debugGroupStartID( layer );
 
    /* get the appropriate layer */
    switch ( layer ) {
@@ -1210,11 +1223,12 @@ void spfx_render( int layer, double dt )
       break;
    }
 
+   gl_debugGroupEnd();
    NTracingZoneEnd( _ctx );
 }
 
 /**
- * @brief Parses raw values out of a "trail" element.
+ * @brief Parses raw values out of a `trail` element.
  * \warning This means values like idle->thick aren't ready to use.
  */
 static int trailSpec_parse( TrailSpec *tc, const char *file, int firstpass )
@@ -1288,6 +1302,8 @@ static int trailSpec_parse( TrailSpec *tc, const char *file, int firstpass )
          tc->nebula = xml_getInt( node );
       else if ( xml_isNode( node, "accel_mod" ) )
          tc->accel_mod = xml_getFloat( node );
+      else if ( xml_isNode( node, "accel_base" ) )
+         tc->accel_base = xml_getFloat( node );
       else {
          int i;
          for ( i = 0; i < MODE_MAX; i++ )
