@@ -799,7 +799,7 @@ void diff_end( void )
 }
 
 /**
- * @brief Parsess the attributes.
+ * @brief Parses the attributes.
  */
 static void diff_parseAttr( UniHunk_t *hunk, xmlNodePtr node )
 {
@@ -1325,7 +1325,7 @@ int diff_patchHunk( UniHunk_t *hunk )
       } else
          WARN( _( "Hunk '%s' can not find an asteroid field with label '%s' in "
                   "system '%s'!" ),
-               hunk->target.type, hunk->u.name, ssys->name );
+               diff_hunkName( hunk->type ), hunk->u.name, ssys->name );
    }
       return -1;
    /* Removing an asteroid field. */
@@ -1338,7 +1338,7 @@ int diff_patchHunk( UniHunk_t *hunk )
       } else
          WARN( _( "Hunk '%s' can not find an asteroid field with label '%s' in "
                   "system '%s'!" ),
-               hunk->target.type, hunk->u.name, ssys->name );
+               diff_hunkName( hunk->type ), hunk->u.name, ssys->name );
    }
       return -1;
    case HUNK_TYPE_SSYS_ASTEROIDS_REMOVE_REVERT:
@@ -1539,7 +1539,7 @@ int diff_patchHunk( UniHunk_t *hunk )
          hunk->o.name = faction_name( p->presence.faction );
       diff_universe_changed = 1;
       /* Special case to clear the faction. */
-      if ( strcasecmp( hunk->u.name, "None" ) == 0 )
+      if ( SDL_strcasecmp( hunk->u.name, "None" ) == 0 )
          return spob_setFaction( p, -1 );
       else
          return spob_setFaction( p, faction_get( hunk->u.name ) );
@@ -1748,6 +1748,8 @@ int diff_patchHunk( UniHunk_t *hunk )
          hunk->o.data = 'A';
       else if ( areEnemies( a, b ) )
          hunk->o.data = 'E';
+      else if ( areNeutral( a, b ) )
+         hunk->o.data = 'N';
       else
          hunk->o.data = 0;
       faction_addEnemy( a, b );
@@ -1763,6 +1765,8 @@ int diff_patchHunk( UniHunk_t *hunk )
          hunk->o.data = 'A';
       else if ( areEnemies( a, b ) )
          hunk->o.data = 'E';
+      else if ( areNeutral( a, b ) )
+         hunk->o.data = 'N';
       else
          hunk->o.data = 0;
       faction_rmAlly( a, b );
@@ -1777,16 +1781,29 @@ int diff_patchHunk( UniHunk_t *hunk )
       if ( b < 0 )
          return -1;
       if ( hunk->o.data == 'A' ) {
+         faction_rmNeutral( a, b );
+         faction_rmNeutral( b, a );
          faction_rmEnemy( a, b );
          faction_rmEnemy( b, a );
          faction_addAlly( a, b );
          faction_addAlly( b, a );
       } else if ( hunk->o.data == 'E' ) {
+         faction_rmNeutral( a, b );
+         faction_rmNeutral( b, a );
          faction_rmAlly( a, b );
          faction_rmAlly( b, a );
          faction_addEnemy( a, b );
          faction_addEnemy( b, a );
+      } else if ( hunk->o.data == 'N' ) {
+         faction_rmAlly( a, b );
+         faction_rmAlly( b, a );
+         faction_rmEnemy( a, b );
+         faction_rmEnemy( b, a );
+         faction_addNeutral( a, b );
+         faction_addNeutral( b, a );
       } else {
+         faction_rmNeutral( a, b );
+         faction_rmNeutral( b, a );
          faction_rmAlly( a, b );
          faction_rmAlly( b, a );
          faction_rmEnemy( a, b );
@@ -2001,7 +2018,17 @@ int diff_load( xmlNodePtr parent )
                            "unidiff. Was empty." ) );
                   continue;
                }
-               diff_applyInternal( diffName, 0 );
+               if ( diff_applyInternal( diffName, 0 ) ) {
+                  if ( player_runUpdaterScript( "unidiff", diffName, 0 ) !=
+                       0 ) {
+                     if ( lua_type( naevL, -1 ) == LUA_TSTRING )
+                        diff_applyInternal( lua_tostring( naevL, -1 ), 0 );
+                     else
+                        WARN( "Invalid value returned from 'unidiff' in "
+                              "'save_updater.lua'" );
+                     lua_pop( naevL, 1 );
+                  }
+               }
                continue;
             }
          } while ( xml_nextNode( cur ) );
